﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CyUSB;

namespace AnalogChannal
{
    static class AnalogChannalCommand
    {

        public enum AnalogCMD : byte
        {
            SelChannal = 0xb1,
            SetCouple = 0xb2,
            SetAttenuator = 0xb3,
            SetGainVoltage = 0xb4,
            SetShiftVoltage = 0xb5,
            SetTriggerVoltage = 0xb1
        }

        public enum AnalogAddress : ushort
        {
            HC595 = 0x00,
            TLV5631 = 0x01,
            TLV5631_CH_BAK = 0x02,
            LED = 0x3,
            END_SRAM_ADDR = 0x4,
            SRAM_DATA_PORT = 0x5,
            SRAM_EMPTY = 0x6,
            SAMPLE_CRTL = 0x7,
            FREQ_CALU_COUNTER = 0x8,
            SAMPLE_LENGTH = 0x9,
            INT_SRCS_PENDING = 0xa,
            INT_MASK = 0xb,
            INT_PENDING = 0xc,
            INT_OFFSET = 0xd,
            USB_CTRL = 0xe,
            HOST_ARM_SEL = 0xf
        }

        public enum Voltage : ushort
        {
            CHAGainVoltage = 0x7 << 12,
            CHAShiftVoltage = 0x6 << 12,
            CHATriggerVoltage = 0x5 << 12,
            
            CHBGainVoltage = 0x0 << 12,
            CHBShiftVoltage = 0x1 << 12,
            CHBTriggerVoltage = 0x2 << 12
        }

        public static bool Command(this CyFX2Device USBDevice, AnalogCMD CommandCode, ushort value)
        {
            CyControlEndPoint CtrlEndPt = USBDevice.ControlEndPt;
            if (CtrlEndPt == null)
                return false;

            CtrlEndPt.Target = CyConst.TGT_DEVICE;
            CtrlEndPt.ReqType = CyConst.REQ_VENDOR;
            CtrlEndPt.Direction = CyConst.DIR_TO_DEVICE;
            CtrlEndPt.ReqCode = (byte)CommandCode;               
            CtrlEndPt.Value = value;
            CtrlEndPt.Index = 1;

            int len = 0;
            byte[] buf = new byte[1];

            return CtrlEndPt.XferData(ref buf, ref len);
        }

        public static bool CommandToAnalog(this CyFX2Device USBDevice, AnalogAddress address, ushort value)
        {
            CyControlEndPoint CtrlEndPt = USBDevice.ControlEndPt;
            if(CtrlEndPt == null)
                return false;

            CtrlEndPt.Target = CyConst.TGT_DEVICE;
            CtrlEndPt.ReqType = CyConst.REQ_VENDOR;
            CtrlEndPt.Direction = CyConst.DIR_TO_DEVICE;
            CtrlEndPt.ReqCode = 0xb1;               
            CtrlEndPt.Value = value;
            CtrlEndPt.Index = (ushort)address;
            
            int len = 0;
            byte[] buf = new byte[1];

            return CtrlEndPt.XferData(ref buf, ref len);
        }

        public static bool CommandFromAnalog(this CyFX2Device USBDevice, AnalogAddress address, ref ushort value)
        {
            CyControlEndPoint CtrlEndPt = USBDevice.ControlEndPt;
            if (CtrlEndPt == null)
                return false;

            CtrlEndPt.Target = CyConst.TGT_DEVICE;
            CtrlEndPt.ReqType = CyConst.REQ_VENDOR;
            CtrlEndPt.Direction = CyConst.DIR_FROM_DEVICE;
            CtrlEndPt.ReqCode = 0xb2;
            CtrlEndPt.Value = value;
            CtrlEndPt.Index = (ushort)address;

            int len = 2;
            byte[] buf = new byte[2];

            bool retVal = CtrlEndPt.XferData(ref buf, ref len);
            
            value = (ushort)((ushort)(buf[1] << 8) | (ushort)buf[0]);
            
            return retVal;
        }
    }
}
