﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CyUSB;

namespace AnalogChannal
{
    public partial class Form1 : Form
    {
        //USB设备对象
        USBDeviceList usbDevicesList;
        CyFX2Device usbDeviceFx2;

        //指示采集启动
        bool IsSampleStart;
        bool IsStarted;
        bool IsTransfered;
        //周期定时器，用于模拟采集线程
        Timer tik;
        //USB批端点，在本设计中EP2被配置为批端点用于采样数据传输
        CyBulkEndPoint epIn;

        //纵向定标画笔
        Pen blackPenDash;
        //横向定标画笔
        Pen blackPen;
        //通道A画笔
        Pen redPen;
        //通道B画笔
        Pen yellowPen;
        //背景画笔
        Brush brushBlue;
        //窗口背景画笔
        Brush brushSystem;

        //画布设备上下文
        Graphics dc;
        //画布
        Bitmap drawMap;

        int Freq_Avg_Cnt;
        float F_K_Total;

        public Form1()
        {
            InitializeComponent();
            
            //创建定时器，并初始化
            tik = new Timer();
            tik.Interval = 30;
            tik.Tick += new System.EventHandler(tikCallback);
            tik.Enabled = true;
            
            //创建画笔
            blackPenDash = new Pen(Color.White, 1);
            blackPenDash.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;

            blackPen = new Pen(Color.White, 1);
            redPen = new Pen(Color.Red, 1);
            yellowPen = new Pen(Color.Yellow, 1);
            brushBlue = new SolidBrush(Color.Blue);
            brushSystem = new SolidBrush(this.BackColor);

            //创建画布及画布设备上下文
            drawMap = new Bitmap(830, 260);
            dc = Graphics.FromImage(drawMap);

            //设置窗口重绘模式
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer |
                    ControlStyles.ResizeRedraw |
                    ControlStyles.AllPaintingInWmPaint, true);

            //使用高质量图像平滑
            dc.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            //使用高质量像素偏移
            dc.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality; 

            IsStarted = false;
            Freq_Avg_Cnt = 0;
            F_K_Total = 0;

            //初始化设备
            InitDevice();
        }
        
        private void InitDevice()
        {
            usbDevicesList = new USBDeviceList(CyConst.DEVICES_CYUSB);
            usbDevicesList.DeviceAttached += new EventHandler(usbDevicesList_DeviceChange);
            usbDevicesList.DeviceRemoved += new EventHandler(usbDevicesList_DeviceChange);
            usbDevicesList_DeviceChange(null, null);
        }

        void usbDevicesList_DeviceChange(object sender, EventArgs e)
        {

            if (usbDevicesList.Count != 0)
            {              
                //当检测到设备连接时使能下载固件按钮
                usbDeviceFx2 = usbDevicesList[0] as CyFX2Device;

                labelConnect.Text = "已连接";
                labelConnect.ForeColor = System.Drawing.Color.Green;

                if (usbDeviceFx2.Product != "")
                {
                    //当固件下载成功，设备成功枚举后使能窗口控件并绘制波形显示背景
                    groupCHA.Enabled = true;
                    groupCHB.Enabled = true;

                    groupBox3.Enabled = true;
                    groupTrigger.Enabled = true;
                    ButtonSampleStart.Enabled = true;
                    buttonSampleStop.Enabled = true;

                    buttonLoadFirmware.Enabled = false;

                    usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.HOST_ARM_SEL, (ushort)0x1);
                    comboBox1.SelectedIndex = 0;
                    ushort value = 0;
                    usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);

                    ushort adc_dfs = 0x2000;
                    value &= (ushort)(~adc_dfs);
                    usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);

                    value = 0x3ff;
                    usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.END_SRAM_ADDR, value);

                    labelFirmwareLoaded.Text = "已装载";
                    labelFirmwareLoaded.ForeColor = System.Drawing.Color.Green;

                    labelDeviceName.Text = usbDeviceFx2.Product;
                    labelDeviceName.ForeColor = System.Drawing.Color.Green;

                    epIn = usbDeviceFx2.EndPoints[1] as CyBulkEndPoint;
                    DrawBack();
                    DrawCoordinate();
                    Invalidate();
                }
                else
                {
                    groupCHA.Enabled = false;
                    groupCHB.Enabled = false;
                    groupBox3.Enabled = false;
                    groupTrigger.Enabled = false;
                    ButtonSampleStart.Enabled = false;
                    buttonSampleStop.Enabled = false;
                    buttonLoadFirmware.Enabled = true;
                    dc.FillRectangle(brushSystem, 5, 5, 815, 255);
                    Invalidate();
                }

                
            }
            else
            {
                //当设备断开连接时，除能所有窗口控件，并擦除波形显示背景，恢复其至窗口背景色
                labelConnect.Text = "未连接";
                labelConnect.ForeColor = System.Drawing.Color.Red;

                labelDeviceName.Text = "未知";
                labelDeviceName.ForeColor = System.Drawing.Color.Red;

                labelFirmwareLoaded.Text = "未装载";
                labelFirmwareLoaded.ForeColor = System.Drawing.Color.Red;

                buttonLoadFirmware.Enabled = false;

                groupCHA.Enabled = false;
                groupCHB.Enabled = false;
                groupBox3.Enabled = false;
                groupTrigger.Enabled = false;
                ButtonSampleStart.Enabled = false;
                buttonSampleStop.Enabled = false;
                dc.FillRectangle(brushSystem, 5, 5, 815, 255);
                Invalidate();
            }

            radioButtonCHADC.Checked = true;

            trackBarCHAGainVoltage.Value = 0;
            trackBarCHAGainVoltage2.Value = 0;

            trackBarCHAShiftVoltage.Value = 0;
            trackBarCHAShiftVoltage2.Value = 0;

            trackBarCHATriggerVoltage.Value = 0;
            trackBarCHATriggerVoltage2.Value = 0;

            radioButtonCHAX100.Checked = true;

            radioButtonCHBDC.Checked = true;

            trackBarCHBGainVoltage.Value = 0;
            trackBarCHBGainVoltage2.Value = 0;

            trackBarCHBShiftVoltage.Value = 0;
            trackBarCHBShiftVoltage2.Value = 0;

            trackBarCHBTriggerVoltage.Value = 0;
            trackBarCHBTriggerVoltage2.Value = 0;

            radioButtonCHBX100.Checked = true;

            labelCommand.Text = "NOP";
            labelCommandReturn.Text = "NOP";

            labelCommand.ForeColor = System.Drawing.Color.Blue;
            labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
        }

        private void buttonLoadFirmware_Click(object sender, EventArgs e)
        {
            //装载固件事件处理
            groupCHA.Enabled = false;
            groupCHB.Enabled = false;
            groupBox3.Enabled = false;
            groupTrigger.Enabled = false;
            ButtonSampleStart.Enabled = false;
            buttonSampleStop.Enabled = false;
            dc.FillRectangle(brushSystem, 5, 5, 815, 255);
            Invalidate();

            labelCommand.Text = "装载固件";

            if (!usbDeviceFx2.LoadRAM("Firmware.hex"))
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }
            else
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
          
            usbDevicesList_DeviceChange(null, null);
        }

        private void trackBarCHAGainVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHAGainVoltage.Value;
            int MinorValue = trackBarCHAGainVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if((string)bar.Tag == "1")
            {
                trackBarCHAGainVoltage2.Value = 0;
                MinorValue = 0;
                trackBarCHAGainVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHAGainVoltage | ((MajorValue * 32 + MinorValue) << 2);
            CHAGainVoltage.Text = (MajorValue * 64 + MinorValue * 2 - 1024).ToString();

            labelCommand.Text = "通道A增益电压:" + CHAGainVoltage.Text + "毫伏";
            if(usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }  
        }

        private void trackBarCHAShiftVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHAShiftVoltage.Value;
            int MinorValue = trackBarCHAShiftVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if ((string)bar.Tag == "1")
            {
                trackBarCHAShiftVoltage2.Value = -31;
                MinorValue = -31;
                trackBarCHAShiftVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHAShiftVoltage | ((-MajorValue * 32 + -MinorValue) << 2);
            CHAShiftVoltage.Text = (1024 + MajorValue * 64 + MinorValue * 2).ToString();

            labelCommand.Text = "通道A移位电压:" + CHAShiftVoltage.Text + "毫伏";
            if (usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }  
        }

        private void trackBarCHATriggerVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHATriggerVoltage.Value;
            int MinorValue = trackBarCHATriggerVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if ((string)bar.Tag == "1")
            {
                trackBarCHATriggerVoltage2.Value = 0;
                MinorValue = 0;
                trackBarCHATriggerVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHATriggerVoltage | ((MajorValue * 32 + MinorValue) << 2);
            CHATriggerVoltage.Text = (MajorValue * 64 + MinorValue * 2).ToString();

            labelCommand.Text = "通道A触发电压:" + CHATriggerVoltage.Text + "毫伏";
            if (usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }  
        }

        private void trackBarCHBGainVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHBGainVoltage.Value;
            int MinorValue = trackBarCHBGainVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if ((string)bar.Tag == "1")
            {
                trackBarCHBGainVoltage2.Value = 0;
                MinorValue = 0;
                trackBarCHBGainVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHBGainVoltage | ((MajorValue * 32 + MinorValue) << 2);
            CHBGainVoltage.Text = (MajorValue * 64 + MinorValue * 2 - 1024).ToString();

            labelCommand.Text = "通道B增益电压:" + CHBGainVoltage.Text + "毫伏";
            if (usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }  
        }

        private void trackBarCHBShiftVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHBShiftVoltage.Value;
            int MinorValue = trackBarCHBShiftVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if ((string)bar.Tag == "1")
            {
                trackBarCHBShiftVoltage2.Value = -31;
                MinorValue = -31;
                trackBarCHBShiftVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHBShiftVoltage | ((-MajorValue * 32 + -MinorValue) << 2);
            CHBShiftVoltage.Text = (1024 + MajorValue * 64 + MinorValue * 2).ToString();

            labelCommand.Text = "通道B移位电压:" + CHBShiftVoltage.Text + "毫伏";
            if (usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            } 
        }

        private void trackBarCHBTriggerVoltage_ValueChanged(object sender, EventArgs e)
        {
            int MajorValue = trackBarCHBTriggerVoltage.Value;
            int MinorValue = trackBarCHBTriggerVoltage2.Value;
            int Value;

            TrackBar bar = sender as TrackBar;

            if ((string)bar.Tag == "1")
            {
                trackBarCHBTriggerVoltage2.Value = 0;
                MinorValue = 0;
                trackBarCHBTriggerVoltage2.Enabled = bar.Value == 32 ? false : true;
            }

            Value = (ushort)AnalogChannalCommand.Voltage.CHBTriggerVoltage | ((MajorValue * 32 + MinorValue) << 2);
            CHBTriggerVoltage.Text = (MajorValue * 64 + MinorValue * 2).ToString();

            labelCommand.Text = "通道B触发电压:" + CHBTriggerVoltage.Text + "毫伏";
            if (usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.TLV5631, (ushort)Value))
            {
                labelCommandReturn.Text = "成功";
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
            }
            else
            {
                labelCommandReturn.Text = "失败";
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
            }
        }

        private void radioButtons_CheckedChanged(object sender, EventArgs e)
        {
            ushort CHACouple = 0;
            ushort CHAAtte = 0;
            ushort CHBCouple = 0;
            ushort CHBAtte = 0;

            CHACouple = (ushort)(radioButtonCHADC.Checked ? 0 : 1);
            CHBCouple = (ushort)(radioButtonCHBDC.Checked ? 0 : 1);

            if(radioButtonCHAX1.Checked)
                CHAAtte = 0x2;
            
            if(radioButtonCHAX10.Checked)
                CHAAtte = 0x1;
            
            if(radioButtonCHAX100.Checked)
                CHAAtte = 0x0;

            if (radioButtonCHBX1.Checked)
                CHBAtte = 0x2;

            if (radioButtonCHBX10.Checked)
                CHBAtte = 0x1;

            if (radioButtonCHBX100.Checked)
                CHBAtte = 0x0;
            
            ushort value = 0;

            value = (ushort)((CHBAtte << 6) | (CHBCouple << 5) | (CHAAtte << 3) | (CHACouple << 2));

            labelCommand.Text = "改变通道衰减耦合";

            if(usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.HC595, value))
            {
                labelCommandReturn.ForeColor = System.Drawing.Color.Blue;
                labelCommandReturn.Text = "成功";
            }
            else
            {
                labelCommandReturn.ForeColor = System.Drawing.Color.Red;
                labelCommandReturn.Text = "失败";
            }
        }

        private void label16_Click(object sender, EventArgs e)
        {

        }

        private void labelFirmwareLoaded_Click(object sender, EventArgs e)
        {

        }

        private void buttonSampleStop_Click(object sender, EventArgs e)
        {            
            IsSampleStart = false;            
            labelSampleStart.Text = "采样停止";
            labelSampleStart.ForeColor = System.Drawing.Color.Red;
        }

        private void ButtonSampleStart_Click(object sender, EventArgs e)
        {
            IsSampleStart = true;
            labelSampleStart.Text = "采样中...";
            labelSampleStart.ForeColor = System.Drawing.Color.Green;
        }

        private void radioSampleFreqAuto_CheckedChanged(object sender, EventArgs e)
        {
            ushort value = 0;
            ushort freqAuto = 0x80;
            usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);

            if(radioSampleFreqAuto.Checked){
                value |= freqAuto;
                comboBox1.Enabled = false;
            }

            if(radioSampleFreqManual.Checked){
                value &= (ushort)(~freqAuto);
                comboBox1.Enabled = true;
            }

            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);   

            comboBox1_SelectedIndexChanged(sender, e);
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            ushort value = 0;
            ushort freqSelMask = 0xf;

            usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
            ushort freqSel = (ushort)(comboBox1.SelectedIndex);

            value &= (ushort)(~freqSelMask);
            value |= freqSel;

            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);
        }

        private void checkTriggerON_CheckedChanged(object sender, EventArgs e)
        {
            ushort value = 0;
            ushort triggerOnMask = 0x1000;
            usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
            
            value &= (ushort)(~triggerOnMask);

            if(checkTriggerON.Checked)
                value |= triggerOnMask;

            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);
        }

        private void radioTriggerSrcCHA_CheckedChanged(object sender, EventArgs e)
        {
            ushort value = 0;
            ushort triggerSelMask = 0x800;

            usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
            value &= (ushort)(~triggerSelMask);

            if(radioTriggerSrcCHA.Checked)
                value |= triggerSelMask;

            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);
        }

        private void radioTriggerRaising_CheckedChanged(object sender, EventArgs e)
        {
            ushort value = 0;
            ushort triggerTypeMask = 0x400;

            usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
            value &= (ushort)(~triggerTypeMask);

            if (radioTriggerRaising.Checked)
                value |= triggerTypeMask;

            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);
        }

        private void tikCallback(Object sender, EventArgs e)
        {
            ushort value = 0;
            int len = 0x800;
            byte[] buf = new byte[len];

            ushort freq_calu_counter = 0;

            if(IsSampleStart){
                if(!IsStarted){
                    usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
                    value |= 0x200;
                    usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, value);
                    IsTransfered = false;
                    IsStarted = true;
                }                 


                if(IsTransfered)
                    IsStarted = false;
                else{
                    usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.SAMPLE_CRTL, ref value);
                    if ((value & 0x4000) != 0){                    
                        usbDeviceFx2.CommandFromAnalog(AnalogChannalCommand.AnalogAddress.FREQ_CALU_COUNTER, ref freq_calu_counter);
                        Freq_Display(freq_calu_counter, (ushort)(value & 0xf));
                        DrawBack();
                        GetData(ref buf, ref len);
                        DrawSignal(ref buf);
                        DrawCoordinate();
                        IsTransfered = true;
                        Invalidate();
                    }
                }
            }
        }

        private void GetData(ref byte[] buf, ref int len)
        {
            ushort value = 1;
            usbDeviceFx2.CommandToAnalog(AnalogChannalCommand.AnalogAddress.USB_CTRL, value);
            epIn.XferData(ref buf, ref len);
        }

        private void DrawBack()
        {
            dc.FillRectangle(brushBlue, 5, 5, 815, 255);
        }

        private void DrawCoordinate()
        {
            dc.DrawLine(blackPen, 5, 135, 818, 135);
            dc.DrawLine(blackPen, 410, 255, 410, 5);

            float Detal = 100;
            float detal = 10;
            float DetalY = 25;
            for(float x = 10; x < 815; x += detal)
                dc.DrawLine(blackPen, x, 125, x, 135);

            for(float x = 10; x < 815; x += Detal)
                dc.DrawLine(blackPen, x, 120, x, 135);

            for(float y = 135; y > 5; y -= DetalY)
                dc.DrawLine(blackPenDash, 5, y, 815, y);

            for (float y = 135; y < 255; y += DetalY)
                dc.DrawLine(blackPenDash, 5, y, 815, y);
        }

        private void DrawSignal(ref byte[] signalBuf)
        {
            float yMax = 810;
            float y0 = 0;
            float y1 = 0;

            for(int i = 0; i < yMax; i++)
            {
                y0 = 135 - (signalBuf[i * 2] - 0x80);
                y1 = 135 - (signalBuf[(i + 1) * 2] - 0x80);
                dc.DrawLine(redPen, i + 5, y0, i + 6, y1);

                y0 = 135 - (signalBuf[i * 2 + 1] - 0x80);
                y1 = 135 - (signalBuf[(i + 1) * 2 + 1] - 0x80);
                dc.DrawLine(yellowPen, i + 5, y0, i + 6, y1);
            }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawImage(drawMap, 0, 0);
        }

        private void Freq_Display(ushort freq_counter, ushort freq_sel)
        {
            float F_K_Avg = 0;

            float T = (float)(System.Math.Pow(2, freq_sel)) * 10;
            float total_ns =  T * freq_counter;
            float Signal_T = total_ns / (float)254.9625;
            float F_K = 1 / (Signal_T / 1000000);

            labelSampleFreq.Text = radioSampleFreqAuto.Checked ? comboBox1.Items[freq_sel].ToString() : comboBox1.Items[comboBox1.SelectedIndex].ToString();
            labelFreqMode.Text = radioSampleFreqAuto.Checked ? "自动" : "手动";

            Freq_Avg_Cnt++;
            F_K_Total += F_K;

            if(Freq_Avg_Cnt == 5){
                F_K_Avg = F_K_Total / Freq_Avg_Cnt;
                labelFreq.Text = "  " + F_K_Avg.ToString();
                Freq_Avg_Cnt = 0;
                F_K_Total = 0;
            }
        }
    }
}
